@@ -1,30 +1,52 @@
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 
-import os, shutil, traceback, functools, sys
+import functools
+import os
+import shutil
+import sys
+import traceback
 from collections import defaultdict
 from itertools import chain, repeat
 
-from calibre.customize import (CatalogPlugin, FileTypePlugin, PluginNotFound,
-                              MetadataReaderPlugin, MetadataWriterPlugin,
-                              InterfaceActionBase as InterfaceAction,
-                              PreferencesPlugin, platform, InvalidPlugin,
-                              StoreBase as Store, EditBookToolPlugin,
-                              LibraryClosedPlugin, PluginInstallationType)
-from calibre.customize.conversion import InputFormatPlugin, OutputFormatPlugin
-from calibre.customize.zipplugin import loader
-from calibre.customize.profiles import InputProfile, OutputProfile
+from calibre.constants import DEBUG, ismacos, numeric_version, system_plugins_loc
+from calibre.customize import (
+    CatalogPlugin,
+    EditBookToolPlugin,
+    FileTypePlugin,
+    InvalidPlugin,
+    LibraryClosedPlugin,
+    MetadataReaderPlugin,
+    MetadataWriterPlugin,
+    PluginInstallationType,
+    PluginNotFound,
+    PreferencesPlugin,
+    platform,
+)
+from calibre.customize import InterfaceActionBase as InterfaceAction
+from calibre.customize import StoreBase as Store
 from calibre.customize.builtins import plugins as builtin_plugins
+from calibre.customize.conversion import InputFormatPlugin, OutputFormatPlugin
+from calibre.customize.profiles import InputProfile, OutputProfile
+from calibre.customize.zipplugin import loader
 from calibre.devices.interface import DevicePlugin
 from calibre.ebooks.metadata import MetaInformation
-from calibre.utils.config import (make_config_dir, Config, ConfigProxy,
-                                 plugin_dir, OptionParser)
 from calibre.ebooks.metadata.sources.base import Source
-from calibre.constants import DEBUG, numeric_version, system_plugins_loc
+from calibre.utils.config import Config, ConfigProxy, OptionParser, make_config_dir, plugin_dir
 from polyglot.builtins import iteritems, itervalues
 
 builtin_names = frozenset(p.name for p in builtin_plugins)
-BLACKLISTED_PLUGINS = frozenset({'Marvin XD', 'iOS reader applications'})
+BLACKLISTED_PLUGINS = frozenset({
+    'Marvin XD',
+    'iOS reader applications',
+
+    # Subsumed by builtin functionality
+    'KoboTouchExtended',
+    'KePub Input',
+    'KePub Output',
+    'KePub Metadata Reader',
+    'KePub Metadata Writer',
+})
 
 
 def zip_value(iterable, value):
@@ -62,24 +84,29 @@ def load_plugin(path_to_zip_file):  # {{{
     :return: A :class:`Plugin` instance.
     '''
     return loader.load(path_to_zip_file)
-
 # }}}
 
+
 # Enable/disable plugins {{{
 
+def disable_plugin_by_name(name: str) -> None:
+    dp = config['disabled_plugins']
+    dp.add(name)
+    config['disabled_plugins'] = dp
+    ep = config['enabled_plugins']
+    if name in ep:
+        ep.remove(name)
+        config['enabled_plugins'] = ep
+
 
 def disable_plugin(plugin_or_name):
     x = getattr(plugin_or_name, 'name', plugin_or_name)
     plugin = find_plugin(x)
+    if plugin is None:
+        raise ValueError(f'No plugin named: {x} found')
     if not plugin.can_be_disabled:
-        raise ValueError('Plugin %s cannot be disabled'%x)
-    dp = config['disabled_plugins']
-    dp.add(x)
-    config['disabled_plugins'] = dp
-    ep = config['enabled_plugins']
-    if x in ep:
-        ep.remove(x)
-    config['enabled_plugins'] = ep
+        raise ValueError(f'Plugin {x} cannot be disabled')
+    disable_plugin_by_name(x)
 
 
 def enable_plugin(plugin_or_name):
@@ -110,39 +137,50 @@ default_disabled_plugins = {
 }
 
 
-def is_disabled(plugin):
-    if plugin.name in config['enabled_plugins']:
+def is_disabled(plugin_or_name):
+    name = getattr(plugin_or_name, 'name', plugin_or_name)
+    if name in config['enabled_plugins']:
         return False
-    return plugin.name in config['disabled_plugins'] or \
-            plugin.name in default_disabled_plugins
+    return name in config['disabled_plugins'] or name in default_disabled_plugins
 # }}}
 
+
 # File type plugins {{{
 
-
 _on_import           = {}
 _on_postimport       = {}
+_on_postconvert      = {}
+_on_postdelete       = {}
 _on_preprocess       = {}
 _on_postprocess      = {}
 _on_postadd          = []
 
 
 def reread_filetype_plugins():
-    global _on_import, _on_postimport, _on_preprocess, _on_postprocess, _on_postadd
+    global _on_import, _on_postimport, _on_postconvert, _on_postdelete, _on_preprocess, _on_postprocess, _on_postadd
     _on_import           = defaultdict(list)
     _on_postimport       = defaultdict(list)
+    _on_postconvert      = defaultdict(list)
+    _on_postdelete       = defaultdict(list)
     _on_preprocess       = defaultdict(list)
     _on_postprocess      = defaultdict(list)
     _on_postadd          = []
 
     for plugin in _initialized_plugins:
         if isinstance(plugin, FileTypePlugin):
+            if ismacos and plugin.name == 'DeDRM' and plugin.version < (10, 0, 3):
+                print(f'Blacklisting the {plugin.name} plugin as it is too old and causes crashes', file=sys.stderr)
+                continue
             for ft in plugin.file_types:
                 if plugin.on_import:
                     _on_import[ft].append(plugin)
                 if plugin.on_postimport:
                     _on_postimport[ft].append(plugin)
                     _on_postadd.append(plugin)
+                if plugin.on_postconvert:
+                    _on_postconvert[ft].append(plugin)
+                if plugin.on_postdelete:
+                    _on_postdelete[ft].append(plugin)
                 if plugin.on_preprocess:
                     _on_preprocess[ft].append(plugin)
                 if plugin.on_postprocess:
@@ -152,6 +190,7 @@ def reread_filetype_plugins():
 def plugins_for_ft(ft, occasion):
     op = {
         'import':_on_import, 'preprocess':_on_preprocess, 'postprocess':_on_postprocess, 'postimport':_on_postimport,
+        'postconvert':_on_postconvert, 'postdelete':_on_postdelete,
     }[occasion]
     for p in chain(op.get(ft, ()), op.get('*', ())):
         if not is_disabled(p):
@@ -173,11 +212,12 @@ def _run_filetype_plugins(path_to_file, ft=None, occasion='preprocess'):
                 pass
             try:
                 nfp = plugin.run(nfp) or nfp
-            except:
-                print('Running file type plugin %s failed with traceback:'%plugin.name, file=oe)
+            except Exception:
+                print(f'Running file type plugin {plugin.name} failed with traceback:', file=oe)
                 traceback.print_exc(file=oe)
         sys.stdout, sys.stderr = oo, oe
-    x = lambda j: os.path.normpath(os.path.normcase(j))
+    def x(j):
+        return os.path.normpath(os.path.normcase(j))
     if occasion == 'postprocess' and x(nfp) != x(path_to_file):
         shutil.copyfile(nfp, path_to_file)
         nfp = path_to_file
@@ -197,9 +237,34 @@ def run_plugins_on_postimport(db, book_id, fmt):
         with plugin:
             try:
                 plugin.postimport(book_id, fmt, db)
-            except:
-                print('Running file type plugin %s failed with traceback:'%
-                       plugin.name)
+            except Exception:
+                print(f'Running file type plugin {plugin.name} failed with traceback:', file=sys.stderr)
+                traceback.print_exc()
+
+
+def run_plugins_on_postconvert(db, book_id, fmt):
+    customization = config['plugin_customization']
+    fmt = fmt.lower()
+    for plugin in plugins_for_ft(fmt, 'postconvert'):
+        plugin.site_customization = customization.get(plugin.name, '')
+        with plugin:
+            try:
+                plugin.postconvert(book_id, fmt, db)
+            except Exception:
+                print(f'Running file type plugin {plugin.name} failed with traceback:', file=sys.stderr)
+                traceback.print_exc()
+
+
+def run_plugins_on_postdelete(db, book_id, fmt):
+    customization = config['plugin_customization']
+    fmt = fmt.lower()
+    for plugin in plugins_for_ft(fmt, 'postdelete'):
+        plugin.site_customization = customization.get(plugin.name, '')
+        with plugin:
+            try:
+                plugin.postdelete(book_id, fmt, db)
+            except Exception:
+                print(f'Running file type plugin {plugin.name} failed with traceback:', file=sys.stderr)
                 traceback.print_exc()
 
 
@@ -213,14 +278,13 @@ def run_plugins_on_postadd(db, book_id, fmt_map):
             try:
                 plugin.postadd(book_id, fmt_map, db)
             except Exception:
-                print('Running file type plugin %s failed with traceback:'%
-                       plugin.name)
+                print(f'Running file type plugin {plugin.name} failed with traceback:', file=sys.stderr)
                 traceback.print_exc()
 
 # }}}
 
-# Plugin customization {{{
 
+# Plugin customization {{{
 
 def customize_plugin(plugin, custom):
     d = config['plugin_customization']
@@ -233,8 +297,8 @@ def plugin_customization(plugin):
 
 # }}}
 
-# Input/Output profiles {{{
 
+# Input/Output profiles {{{
 
 def input_profiles():
     for plugin in _initialized_plugins:
@@ -248,8 +312,8 @@ def output_profiles():
             yield plugin
 # }}}
 
-# Interface Actions # {{{
 
+# Interface Actions # {{{
 
 def interface_actions():
     customization = config['plugin_customization']
@@ -260,8 +324,8 @@ def interface_actions():
                 yield plugin
 # }}}
 
-# Preferences Plugins # {{{
 
+# Preferences Plugins # {{{
 
 def preferences_plugins():
     customization = config['plugin_customization']
@@ -272,8 +336,8 @@ def preferences_plugins():
                 yield plugin
 # }}}
 
-# Library Closed Plugins # {{{
 
+# Library Closed Plugins # {{{
 
 def available_library_closed_plugins():
     customization = config['plugin_customization']
@@ -292,8 +356,8 @@ def has_library_closed_plugins():
     return False
 # }}}
 
-# Store Plugins # {{{
 
+# Store Plugins # {{{
 
 def store_plugins():
     customization = config['plugin_customization']
@@ -324,8 +388,8 @@ def available_stores():
 
 # }}}
 
-# Metadata read/write {{{
 
+# Metadata read/write {{{
 
 _metadata_readers = {}
 _metadata_writers = {}
@@ -431,7 +495,7 @@ def get_file_type_metadata(stream, ftype):
                             stream.seek(0)
                         mi = plugin.get_metadata(stream, ftype.lower().strip())
                         break
-                    except:
+                    except Exception:
                         traceback.print_exc()
                         continue
     return mi
@@ -450,7 +514,7 @@ def set_file_type_metadata(stream, mi, ftype, report_error=None):
                         plugin.site_customization = customization.get(plugin.name, '')
                         plugin.set_metadata(stream, mi, ftype.lower().strip())
                         break
-                    except:
+                    except Exception:
                         if report_error is None:
                             from calibre import prints
                             prints('Failed to set metadata for the', ftype.upper(), 'format of:', getattr(mi, 'title', ''), file=sys.stderr)
@@ -468,18 +532,18 @@ def can_set_metadata(ftype):
 
 # }}}
 
-# Add/remove plugins {{{
 
+# Add/remove plugins {{{
 
 def add_plugin(path_to_zip_file):
     make_config_dir()
     plugin = load_plugin(path_to_zip_file)
     if plugin.name in builtin_names:
         raise NameConflict(
-            'A builtin plugin with the name %r already exists' % plugin.name)
+            f'A builtin plugin with the name {plugin.name!r} already exists')
     if plugin.name in get_system_plugins():
         raise NameConflict(
-            'A system plugin with the name %r already exists' % plugin.name)
+            f'A system plugin with the name {plugin.name!r} already exists')
     plugin = initialize_plugin(plugin, path_to_zip_file, PluginInstallationType.EXTERNAL)
     plugins = config['plugins']
     zfp = os.path.join(plugin_dir, plugin.name+'.zip')
@@ -505,7 +569,7 @@ def remove_plugin(plugin_or_name):
             zfp = plugins[name]
             if os.path.exists(zfp):
                 os.remove(zfp)
-        except:
+        except Exception:
             pass
         plugins.pop(name)
     config['plugins'] = plugins
@@ -514,8 +578,8 @@ def remove_plugin(plugin_or_name):
 
 # }}}
 
-# Input/Output format plugins {{{
 
+# Input/Output format plugins {{{
 
 def input_format_plugins():
     for plugin in _initialized_plugins:
@@ -572,8 +636,8 @@ def available_output_formats():
 
 # }}}
 
-# Catalog plugins {{{
 
+# Catalog plugins {{{
 
 def catalog_plugins():
     for plugin in _initialized_plugins:
@@ -597,8 +661,8 @@ def plugin_for_catalog_format(fmt):
 
 # }}}
 
-# Device plugins {{{
 
+# Device plugins {{{
 
 def device_plugins(include_disabled=False):
     for plugin in _initialized_plugins:
@@ -611,6 +675,13 @@ def device_plugins(include_disabled=False):
                     yield plugin
 
 
+def usbms_plugins(include_disabled=True):
+    from calibre.devices.usbms.driver import USBMS
+    for plugin in device_plugins(include_disabled):
+        if isinstance(plugin, USBMS) and plugin.name not in ('Folder Device Interface', 'User Defined USB driver'):
+            yield plugin
+
+
 def disabled_device_plugins():
     for plugin in _initialized_plugins:
         if isinstance(plugin, DevicePlugin):
@@ -619,8 +690,8 @@ def disabled_device_plugins():
                     yield plugin
 # }}}
 
-# Metadata sources2 {{{
 
+# Metadata sources2 {{{
 
 def metadata_plugins(capabilities):
     capabilities = frozenset(capabilities)
@@ -644,15 +715,15 @@ def patch_metadata_plugins(possibly_updated_plugins):
             if pup is not None:
                 if pup.version > plugin.version and pup.minimum_calibre_version <= numeric_version:
                     patches[i] = pup(None)
-                    # Metadata source plugins dont use initialize() but that
+                    # Metadata source plugins don't use initialize() but that
                     # might change in the future, so be safe.
                     patches[i].initialize()
     for i, pup in iteritems(patches):
         _initialized_plugins[i] = pup
 # }}}
 
-# Editor plugins {{{
 
+# Editor plugins {{{
 
 def all_edit_book_tool_plugins():
     for plugin in _initialized_plugins:
@@ -660,13 +731,13 @@ def all_edit_book_tool_plugins():
             yield plugin
 # }}}
 
-# Initialize plugins {{{
 
+# Initialize plugins {{{
 
 _initialized_plugins = []
 
 
-def initialize_plugin(plugin, path_to_zip_file, installation_type):
+def initialize_plugin(plugin, path_to_zip_file=None, installation_type=PluginInstallationType.BUILTIN):
     try:
         p = plugin(path_to_zip_file)
         p.installation_type = installation_type
@@ -715,14 +786,23 @@ def initialize_plugins(perf=False):
     for p in system_conflicts:
         system_plugins.pop(p, None)
     external_plugins = config['plugins'].copy()
+
+    if 'KoboTouchExtended' in external_plugins and is_disabled('KoboTouch') and not is_disabled('KoboTouchExtended'):
+        # We disable KoboTouchExtended and re-enable KoboTouch so that the Kobo
+        # device keeps working even though KoboTouchExtended is blacklisted.
+        try:
+            disable_plugin_by_name('KoboTouchExtended')
+            enable_plugin('KoboTouch')
+        except Exception:
+            traceback.print_exc()
     for name in BLACKLISTED_PLUGINS:
         external_plugins.pop(name, None)
         system_plugins.pop(name, None)
     ostdout, ostderr = sys.stdout, sys.stderr
     if perf:
-        from collections import defaultdict
         import time
-        times = defaultdict(lambda:0)
+        from collections import defaultdict
+        times = defaultdict(int)
 
     for zfp, installation_type in chain(
             zip_value(external_plugins.items(), PluginInstallationType.EXTERNAL),
@@ -749,8 +829,8 @@ def initialize_plugins(perf=False):
             if perf:
                 times[plugin.name] = time.time() - st
             _initialized_plugins.append(plugin)
-        except:
-            print('Failed to initialize plugin:', repr(zfp))
+        except Exception:
+            print('Failed to initialize plugin:', repr(zfp), file=sys.stderr)
             if DEBUG:
                 traceback.print_exc()
     # Prevent a custom plugin from overriding stdout/stderr as this breaks
@@ -758,7 +838,7 @@ def initialize_plugins(perf=False):
     sys.stdout, sys.stderr = ostdout, ostderr
     if perf:
         for x in sorted(times, key=lambda x: times[x]):
-            print('%50s: %.3f'%(x, times[x]))
+            print(f'{x:50}: {times[x]:.3f}')
     _initialized_plugins.sort(key=lambda x: x.priority, reverse=True)
     reread_filetype_plugins()
     reread_metadata_plugins()
@@ -772,13 +852,13 @@ def initialized_plugins():
 
 # }}}
 
-# CLI {{{
 
+# CLI {{{
 
 def build_plugin(path):
     from calibre import prints
     from calibre.ptempfile import PersistentTemporaryFile
-    from calibre.utils.zipfile import ZipFile, ZIP_STORED
+    from calibre.utils.zipfile import ZIP_STORED, ZipFile
     path = str(path)
     names = frozenset(os.listdir(path))
     if '__init__.py' not in names:
@@ -808,7 +888,8 @@ def option_parser():
     parser.add_option('-r', '--remove-plugin', default=None,
                       help=_('Remove a custom plugin by name. Has no effect on builtin plugins'))
     parser.add_option('--customize-plugin', default=None,
-                      help=_('Customize plugin. Specify name of plugin and customization string separated by a comma.'))
+                      help=_('Customize plugin. Specify name of plugin and customization string separated by a comma.'
+                             ' The customization string is the same as you would enter when customizing the plugin in the main calibre GUI.'))
     parser.add_option('-l', '--list-plugins', default=False, action='store_true',
                       help=_('List all installed plugins'))
     parser.add_option('--enable-plugin', default=None,
@@ -835,10 +916,13 @@ def main(args=sys.argv):
         else:
             print('No custom plugin named', opts.remove_plugin)
     if opts.customize_plugin is not None:
-        name, custom = opts.customize_plugin.split(',')
+        try:
+            name, custom = opts.customize_plugin.split(',')
+        except ValueError:
+            name, custom = opts.customize_plugin, ''
         plugin = find_plugin(name.strip())
         if plugin is None:
-            print('No plugin with the name %s exists'%name)
+            print(f'No plugin with the name {name} exists')
             return 1
         customize_plugin(plugin, custom)
     if opts.enable_plugin is not None:
@@ -849,7 +933,7 @@ def main(args=sys.argv):
         type_len = name_len = 0
         for plugin in initialized_plugins():
             type_len, name_len = max(type_len, len(plugin.type)), max(name_len, len(plugin.name))
-        fmt = '%-{}s%-{}s%-15s%-15s%s'.format(type_len+1, name_len+1)
+        fmt = f'%-{type_len+1}s%-{name_len+1}s%-15s%-15s%s'
         print(fmt%tuple('Type|Name|Version|Disabled|Site Customization'.split('|')))
         print()
         for plugin in initialized_plugins():
