@@ -123,7 +123,7 @@ class Metadata:
             null_val = NULL_VALUES.get(field, None)
             val = getattr(self, field, None)
             return not val or val == null_val
-        except:
+        except Exception:
             return True
 
     def set_null(self, field):
@@ -139,7 +139,7 @@ class Metadata:
         if field == 'language':
             try:
                 return _data.get('languages', [])[0]
-            except:
+            except Exception:
                 return NULL_VALUES['language']
         try:
             return object.__getattribute__(self, field)
@@ -160,7 +160,7 @@ class Metadata:
         if field.startswith('#') and field.endswith('_index'):
             try:
                 return self.get_extra(field[:-6])
-            except:
+            except Exception:
                 pass
         raise AttributeError(
                 'Metadata object has no attribute named: '+ repr(field))
@@ -251,7 +251,7 @@ class Metadata:
         if field in _data['user_metadata']:
             try:
                 return _data['user_metadata'][field]['#extra#']
-            except:
+            except Exception:
                 return default
         raise AttributeError(
                 'Metadata object has no attribute named: '+ repr(field))
@@ -498,7 +498,7 @@ class Metadata:
                     self.set(dest, [f.strip() for f in val.split('&') if f.strip()])
                 else:
                     self.set(dest, val)
-            except:
+            except Exception:
                 if DEBUG:
                     traceback.print_exc()
 
@@ -713,7 +713,7 @@ class Metadata:
                 try:
                     fmt = cmeta['display'].get('number_format', None)
                     res = fmt.format(res)
-                except:
+                except Exception:
                     pass
             return (name, str(res), orig_res, cmeta)
 
@@ -885,3 +885,39 @@ def field_from_string(field, raw, field_metadata):
     if val is object:
         val = raw
     return val
+
+
+def get_model_metadata_instance():
+    '''
+    Get a metadata instance that contains all the fields in the current database
+    with the fields to a plausible value. This function must only be used in
+    the GUI thread.
+    '''
+    from calibre.gui2 import is_gui_thread
+    if not is_gui_thread():
+        raise ValueError('get_model_metadata_instance() must only be used in the GUI thread')
+
+    mi = Metadata(_('Title'), [_('Author')])
+    mi.author_sort = _('Author Sort')
+    mi.series = ngettext('Series', 'Series', 1)
+    mi.series_index = 3
+    mi.rating = 4.0
+    mi.tags = [_('Tag 1'), _('Tag 2')]
+    mi.languages = ['eng']
+    mi.id = -1
+    from calibre.gui2.ui import get_gui
+    from calibre.utils.date import DEFAULT_DATE
+    fm = get_gui().current_db.new_api.field_metadata
+    mi.set_all_user_metadata(fm.custom_field_metadata())
+    for col in mi.get_all_user_metadata(False):
+        if fm[col]['datatype'] == 'datetime':
+            mi.set(col, DEFAULT_DATE)
+        elif fm[col]['datatype'] in ('int', 'float', 'rating'):
+            mi.set(col, 2)
+        elif fm[col]['datatype'] == 'bool':
+            mi.set(col, False)
+        elif fm[col]['is_multiple']:
+            mi.set(col, [col])
+        else:
+            mi.set(col, col, 1)
+    return mi
