@@ -1,25 +1,55 @@
 #!/usr/bin/env python
-# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-
 
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import os, posixpath, weakref, sys
-from functools import partial
+import os
+import posixpath
+import sys
+import weakref
+from contextlib import suppress
+from functools import lru_cache, partial
 
-from qt.core import (QMenu, Qt, QInputDialog, QToolButton, QDialog,
-        QDialogButtonBox, QGridLayout, QLabel, QLineEdit, QIcon, QSize,
-        QCoreApplication, pyqtSignal, QVBoxLayout, QTimer, QAction)
+from qt.core import (
+    QAction,
+    QCoreApplication,
+    QDialog,
+    QDialogButtonBox,
+    QGridLayout,
+    QIcon,
+    QInputDialog,
+    QLabel,
+    QLineEdit,
+    QMenu,
+    QSize,
+    Qt,
+    QTimer,
+    QToolButton,
+    QVBoxLayout,
+    pyqtSignal,
+)
 
 from calibre import isbytestring, sanitize_file_name
-from calibre.constants import (filesystem_encoding, iswindows, get_portable_base, isportable)
+from calibre.constants import config_dir, filesystem_encoding, get_portable_base, isportable, iswindows
+from calibre.gui2 import (
+    Dispatcher,
+    choose_dir,
+    choose_images,
+    error_dialog,
+    gprefs,
+    info_dialog,
+    open_local_file,
+    pixmap_to_data,
+    question_dialog,
+    warning_dialog,
+)
+from calibre.gui2.actions import InterfaceAction
+from calibre.library import current_library_name
+from calibre.startup import connect_lambda
 from calibre.utils.config import prefs, tweaks
 from calibre.utils.icu import sort_key
-from calibre.gui2 import (gprefs, warning_dialog, Dispatcher, error_dialog,
-    question_dialog, info_dialog, open_local_file, choose_dir)
-from calibre.gui2.actions import InterfaceAction
+from calibre.utils.localization import ngettext
 
 
 def db_class():
@@ -27,6 +57,18 @@ def db_class():
     return LibraryDatabase
 
 
+def library_icon_path(lib_name=''):
+    return os.path.join(config_dir, 'library_icons', sanitize_file_name(lib_name or current_library_name()) + '.png')
+
+
+@lru_cache(maxsize=512)
+def library_qicon(lib_name=''):
+    q = library_icon_path(lib_name)
+    if os.path.exists(q):
+        return QIcon(q)
+    return getattr(library_qicon, 'default_icon', None) or QIcon.ic('lt.png')
+
+
 class LibraryUsageStats:  # {{{
 
     def __init__(self):
@@ -79,7 +121,7 @@ class LibraryUsageStats:  # {{{
         if lpath in locs:
             locs.remove(lpath)
         limit = tweaks['many_libraries'] if limit is None else limit
-        key = (lambda x:sort_key(os.path.basename(x))) if len(locs) > limit else self.stats.get
+        key = (lambda x: sort_key(os.path.basename(x))) if len(locs) > limit else self.stats.get
         locs.sort(key=key, reverse=len(locs)<=limit)
         for loc in locs:
             yield self.pretty(loc), loc
@@ -119,14 +161,14 @@ class MovedDialog(QDialog):  # {{{
         self.loc = QLineEdit(loc, self)
         l.addWidget(self.loc, l.rowCount(), 0, 1, 1)
         self.cd = QToolButton(self)
-        self.cd.setIcon(QIcon(I('document_open.png')))
+        self.cd.setIcon(QIcon.ic('document_open.png'))
         self.cd.clicked.connect(self.choose_dir)
         l.addWidget(self.cd, l.rowCount() - 1, 1, 1, 1)
         self.bb = QDialogButtonBox(QDialogButtonBox.StandardButton.Abort)
         b = self.bb.addButton(_('Library moved'), QDialogButtonBox.ButtonRole.AcceptRole)
-        b.setIcon(QIcon(I('ok.png')))
+        b.setIcon(QIcon.ic('ok.png'))
         b = self.bb.addButton(_('Forget library'), QDialogButtonBox.ButtonRole.RejectRole)
-        b.setIcon(QIcon(I('edit-clear.png')))
+        b.setIcon(QIcon.ic('edit-clear.png'))
         b.clicked.connect(self.forget_library)
         self.bb.accepted.connect(self.accept)
         self.bb.rejected.connect(self.reject)
@@ -168,7 +210,7 @@ class BackupStatus(QDialog):  # {{{
         bb.rejected.connect(self.reject)
         b = bb.addButton(_('Queue &all books for backup'), QDialogButtonBox.ButtonRole.ActionRole)
         b.clicked.connect(self.mark_all_dirty)
-        b.setIcon(QIcon(I('lt.png')))
+        b.setIcon(QIcon.ic('lt.png'))
         l.addWidget(bb)
         self.db = weakref.ref(gui.current_db)
         self.setResult(9)
@@ -184,8 +226,8 @@ class BackupStatus(QDialog):  # {{{
             return
         dirty_text = 'no'
         try:
-            dirty_text = '%s' % db.dirty_queue_length()
-        except:
+            dirty_text = f'{db.dirty_queue_length()}'
+        except Exception:
             dirty_text = _('none')
         self.msg.setText('<p>' + _(
             'Book metadata files remaining to be written: %s') % dirty_text)
@@ -244,11 +286,26 @@ class ChooseLibraryAction(InterfaceAction):
             None, None), attr='action_pick_random')
         ac.triggered.connect(self.pick_random)
 
+        self.choose_library_icon_menu = QMenu(_('Change the icon for this library'))
+        self.choose_library_icon_menu.setIcon(QIcon.ic('icon_choose.png'))
+        self.choose_library_icon_action = self.create_action(
+            spec=(_('Choose an icon'), 'icon_choose.png', None, None),
+            attr='action_choose_library_icon')
+        self.remove_library_icon_action = self.create_action(
+            spec=(_('Remove current icon'), 'trash.png', None, None),
+            attr='action_remove_library_icon')
+        self.choose_library_icon_action.triggered.connect(self.get_library_icon)
+        self.remove_library_icon_action.triggered.connect(partial(self.remove_library_icon, ''))
+        self.choose_library_icon_menu.addAction(self.choose_library_icon_action)
+        self.choose_library_icon_menu.addAction(self.remove_library_icon_action)
+        self.original_library_icon = library_qicon.default_icon = self.qaction.icon()
+
         if not os.environ.get('CALIBRE_OVERRIDE_DATABASE_PATH', None):
             self.choose_menu.addAction(self.action_choose)
 
             self.quick_menu = QMenu(_('Quick switch'))
             self.quick_menu_action = self.choose_menu.addMenu(self.quick_menu)
+            self.choose_menu.addMenu(self.choose_library_icon_menu)
             self.rename_menu = QMenu(_('Rename library'))
             self.rename_menu_action = self.choose_menu.addMenu(self.rename_menu)
             self.choose_menu.addAction(ac)
@@ -260,6 +317,7 @@ class ChooseLibraryAction(InterfaceAction):
                                                       type=Qt.ConnectionType.QueuedConnection)
             self.choose_menu.addAction(self.action_exim)
         else:
+            self.choose_menu.addMenu(self.choose_library_icon_menu)
             self.choose_menu.addAction(ac)
 
         self.rename_separator = self.choose_menu.addSeparator()
@@ -267,7 +325,7 @@ class ChooseLibraryAction(InterfaceAction):
         self.switch_actions = []
         for i in range(5):
             ac = self.create_action(spec=('', None, None, None),
-                    attr='switch_action%d'%i)
+                    attr=f'switch_action{i}')
             ac.setObjectName(str(i))
             self.switch_actions.append(ac)
             ac.setVisible(False)
@@ -317,6 +375,53 @@ class ChooseLibraryAction(InterfaceAction):
     def pick_random(self, *args):
         self.gui.iactions['Pick Random Book'].pick_random()
 
+    def get_library_icon(self):
+        try:
+            paths = choose_images(self.gui, 'choose_library_icon',
+                        _('Select icon for library "%s"') % current_library_name())
+            if paths:
+                path = paths[0]
+                p = QIcon(path).pixmap(QSize(256, 256))
+                icp = library_icon_path()
+                os.makedirs(os.path.dirname(icp), exist_ok=True)
+                with open(icp, 'wb') as f:
+                    f.write(pixmap_to_data(p, format='PNG'))
+                self.set_library_icon()
+                library_qicon.cache_clear()
+        except Exception:
+            import traceback
+            traceback.print_exc()
+
+    def rename_library_icon(self, old_name, new_name):
+        old_path = library_icon_path(old_name)
+        new_path = library_icon_path(new_name)
+        try:
+            if os.path.exists(old_path):
+                os.replace(old_path, new_path)
+            library_qicon.cache_clear()
+        except Exception:
+            import traceback
+            traceback.print_exc()
+
+    def remove_library_icon(self, name=''):
+        try:
+            with suppress(FileNotFoundError):
+                os.remove(library_icon_path(name or current_library_name()))
+            self.set_library_icon()
+            library_qicon.cache_clear()
+        except Exception:
+            import traceback
+            traceback.print_exc()
+
+    def set_library_icon(self):
+        icon = QIcon.ic(library_icon_path())
+        has_icon = not icon.isNull() and len(icon.availableSizes()) > 0
+        if not has_icon:
+            icon = self.original_library_icon
+        self.qaction.setIcon(icon)
+        self.gui.setWindowIcon(icon)
+        self.remove_library_icon_action.setEnabled(has_icon)
+
     def exim_data(self):
         if isportable:
             return error_dialog(self.gui, _('Cannot export/import'), _(
@@ -327,7 +432,7 @@ class ChooseLibraryAction(InterfaceAction):
                     _('Cannot export/import data while there are running jobs.'), show=True)
         from calibre.gui2.dialogs.exim import EximDialog
         d = EximDialog(parent=self.gui)
-        if d.exec_() == QDialog.DialogCode.Accepted:
+        if d.exec() == QDialog.DialogCode.Accepted:
             if d.restart_needed:
                 self.gui.iactions['Restart'].restart()
 
@@ -358,6 +463,7 @@ class ChooseLibraryAction(InterfaceAction):
         a.setText(lname.replace('&', '&&&'))  # I have no idea why this requires a triple ampersand
         self.update_tooltip(db.count())
         self.build_menus()
+        self.set_library_icon()
         state = self.view_state_map.get(self.stats.canonicalize_path(
             db.library_path), None)
         if state is not None:
@@ -395,8 +501,9 @@ class ChooseLibraryAction(InterfaceAction):
         quick_actions, rename_actions, delete_actions = [], [], []
         for name, loc in locations:
             is_prev_lib = name == self.prev_lname
+            ic = library_qicon(name)
             name = name.replace('&', '&&')
-            ac = self.quick_menu.addAction(name, Dispatcher(partial(self.switch_requested,
+            ac = self.quick_menu.addAction(ic, name, Dispatcher(partial(self.switch_requested,
                 loc)))
             ac.setStatusTip(_('Switch to: %s') % loc)
             if is_prev_lib:
@@ -421,9 +528,11 @@ class ChooseLibraryAction(InterfaceAction):
             locations_by_frequency = list(self.stats.locations(db, limit=sys.maxsize))
         for i, x in enumerate(locations_by_frequency[:len(self.switch_actions)]):
             name, loc = x
+            ic = library_qicon(name)
             name = name.replace('&', '&&')
             ac = self.switch_actions[i]
             ac.setText(name)
+            ac.setIcon(ic)
             ac.setStatusTip(_('Switch to: %s') % loc)
             ac.setVisible(True)
             qs_actions.append(ac)
@@ -445,7 +554,7 @@ class ChooseLibraryAction(InterfaceAction):
         restrictions.insert(0, '')
         for vl in restrictions:
             if vl == vl_at_startup:
-                self.vl_to_apply_menu.addAction(QIcon(I('ok.png')), vl if vl else _('No Virtual library'),
+                self.vl_to_apply_menu.addAction(QIcon.ic('ok.png'), vl if vl else _('No Virtual library'),
                                                 Dispatcher(partial(self.change_vl_at_startup_requested, vl)))
             else:
                 self.vl_to_apply_menu.addAction(vl if vl else _('No Virtual library'),
@@ -471,7 +580,8 @@ class ChooseLibraryAction(InterfaceAction):
         newname, ok = QInputDialog.getText(self.gui, _('Rename') + ' ' + old_name,
                 '<p>'+_(
                     'Choose a new name for the library <b>%s</b>. ')%name + '<p>'+_(
-                    'Note that the actual library folder will be renamed.'),
+                        'Note that the actual library folder will be renamed.') + '<p>' + _(
+                            'WARNING: This means that any calibre:// URLs that point to things in this library will stop working.'),
                 text=old_name)
         newname = sanitize_file_name(str(newname))
         if not ok or not newname or newname == old_name:
@@ -495,10 +605,9 @@ class ChooseLibraryAction(InterfaceAction):
         self.gui.library_broker.remove_library(loc)
         try:
             os.rename(loc, newloc)
-        except:
+        except Exception:
             import traceback
-            det_msg = 'Location: %r New Location: %r\n%s'%(loc, newloc,
-                                                        traceback.format_exc())
+            det_msg = f'Location: {loc!r} New Location: {newloc!r}\n{traceback.format_exc()}'
             error_dialog(self.gui, _('Rename failed'),
                     _('Failed to rename the library at %s. '
                 'The most common cause for this is if one of the files'
@@ -506,6 +615,7 @@ class ChooseLibraryAction(InterfaceAction):
                     det_msg=det_msg, show=True)
             return
         self.stats.rename(location, newloc)
+        self.rename_library_icon(old_name, newname)
         self.build_menus()
         self.gui.iactions['Copy To Library'].build_menus()
 
@@ -515,10 +625,11 @@ class ChooseLibraryAction(InterfaceAction):
                 self.gui, _('Library removed'), _(
                 'The library %s has been removed from calibre. '
                 'The files remain on your computer, if you want '
-                'to delete them, you will have to do so manually.') % ('<code>%s</code>' % loc),
+                'to delete them, you will have to do so manually.') % (f'<code>{loc}</code>'),
                 override_icon='dialog_information.png',
                 yes_text=_('&OK'), no_text=_('&Undo'), yes_icon='ok.png', no_icon='edit-undo.png'):
             return
+        self.remove_library_icon(name)
         self.stats.remove(location)
         self.gui.library_broker.remove_library(location)
         self.build_menus()
@@ -566,22 +677,37 @@ class ChooseLibraryAction(InterfaceAction):
         db = m.db
         db.prefs.disable_setting = True
         library_path = db.library_path
+        before = db.new_api.size_stats()
 
         d = DBCheck(self.gui, db)
-        d.start()
         try:
-            m.close()
-        except:
-            pass
-        d.break_cycles()
+            d.exec()
+            try:
+                m.close()
+            except Exception:
+                pass
+        finally:
+            d.break_cycles()
         self.gui.library_moved(library_path)
         if d.rejected:
             return
         if d.error is None:
+            after = self.gui.current_db.new_api.size_stats()
+            det_msg = ''
+            from calibre import human_readable
+            for which, title in {'main': _('books'), 'fts': _('full text search'), 'notes': _('notes')}.items():
+                if which != 'main' and not getattr(d, which).isChecked():
+                    continue
+                det_msg += '\n'
+                if before[which] == after[which]:
+                    det_msg += _('Size of the {} database was unchanged.').format(title)
+                else:
+                    det_msg += _('Size of the {0} database reduced from {1} to {2}.').format(
+                            title, human_readable(before[which]), human_readable(after[which]))
             if not question_dialog(self.gui, _('Success'),
                     _('Found no errors in your calibre library database.'
                         ' Do you want calibre to check if the files in your'
-                        ' library match the information in the database?')):
+                        ' library match the information in the database?'), det_msg=det_msg.strip()):
                 return
         else:
             return error_dialog(self.gui, _('Failed'),
@@ -595,9 +721,12 @@ class ChooseLibraryAction(InterfaceAction):
             d = CheckLibraryDialog(self.gui, m.db)
 
             if not d.do_exec():
-                info_dialog(self.gui, _('No problems found'),
-                        _('The files in your library match the information '
-                        'in the database.'), show=True)
+                if question_dialog(self.gui, _('No problems found'),
+                        _('The files in your library match the information in the database.\n\n'
+                          "Choose 'Open dialog' to change settings and run the check again."),
+                        yes_text=_('Open dialog'), yes_icon='gear.png', no_icon='ok.png',
+                        no_text=_('Finished')):
+                    d.exec()
         finally:
             self.gui.status_bar.clear_message()
 
@@ -629,7 +758,7 @@ class ChooseLibraryAction(InterfaceAction):
 
         if not exists:
             d = MovedDialog(self.stats, location, self.gui)
-            ret = d.exec_()
+            ret = d.exec()
             self.build_menus()
             self.gui.iactions['Copy To Library'].build_menus()
             if ret == QDialog.DialogCode.Accepted:
@@ -647,6 +776,7 @@ class ChooseLibraryAction(InterfaceAction):
 
     def debug_leak(self):
         import gc
+
         from calibre.utils.mem import memory
         ref = self.dbref
         for i in range(3):
@@ -673,7 +803,7 @@ class ChooseLibraryAction(InterfaceAction):
         location = self.stats.canonicalize_path(db.library_path)
         self.pre_choose_dialog_location = location
         c = ChooseLibrary(db, self.choose_library_callback, self.gui)
-        c.exec_()
+        c.exec()
 
     def choose_library_callback(self, newloc, copy_structure=False, library_renamed=False):
         self.gui.library_moved(newloc, copy_structure=copy_structure,
