@@ -9,25 +9,42 @@ import os
 from collections import defaultdict
 from contextlib import closing
 from functools import partial
-from qt.core import (
-    QAbstractItemView, QApplication, QCheckBox, QDialog, QDialogButtonBox,
-    QFormLayout, QGridLayout, QHBoxLayout, QIcon, QLabel, QLineEdit, QListWidget,
-    QListWidgetItem, QScrollArea, QSize, Qt, QToolButton, QVBoxLayout, QWidget
-)
 from threading import Thread
 
+from qt.core import (
+    QAbstractItemView,
+    QApplication,
+    QCheckBox,
+    QDialog,
+    QDialogButtonBox,
+    QFormLayout,
+    QGridLayout,
+    QHBoxLayout,
+    QIcon,
+    QLabel,
+    QLineEdit,
+    QListWidget,
+    QListWidgetItem,
+    QScrollArea,
+    QSize,
+    Qt,
+    QToolButton,
+    QVBoxLayout,
+    QWidget,
+)
+
 from calibre import as_unicode
 from calibre.constants import ismacos
 from calibre.db.copy_to_library import copy_one_book
-from calibre.gui2 import (
-    Dispatcher, choose_dir, error_dialog, gprefs, info_dialog, warning_dialog
-)
+from calibre.gui2 import Dispatcher, choose_dir, error_dialog, gprefs, info_dialog, warning_dialog
 from calibre.gui2.actions import InterfaceAction
 from calibre.gui2.actions.choose_library import library_qicon
 from calibre.gui2.dialogs.progress import ProgressDialog
 from calibre.gui2.widgets2 import Dialog
+from calibre.startup import connect_lambda
 from calibre.utils.config import prefs
 from calibre.utils.icu import numeric_sort_key, sort_key
+from calibre.utils.localization import ngettext
 from polyglot.builtins import iteritems, itervalues
 
 
@@ -136,7 +153,7 @@ class Worker(Thread):  # {{{
             import traceback
             try:
                 err = str(err)
-            except:
+            except Exception:
                 err = repr(err)
             self.error = (err, traceback.format_exc())
 
@@ -317,7 +334,6 @@ class DuplicatesQuestion(QDialog):  # {{{
     @property
     def ids(self):
         return {int(i.data(Qt.ItemDataRole.UserRole)) for i in self.items if i.checkState() == Qt.CheckState.Checked}
-
 # }}}
 
 
@@ -456,9 +472,11 @@ class CopyToLibraryAction(InterfaceAction):
 
     def do_copy(self, ids, db, loc, delete_after, add_duplicates=False):
         aname = _('Moving to') if delete_after else _('Copying to')
-        dtitle = '%s %s'%(aname, os.path.basename(loc))
+        dtitle = f'{aname} {os.path.basename(loc)}'
         self.pd = ProgressDialog(dtitle, min=0, max=len(ids)-1,
-                parent=self.gui, cancelable=True, icon='lt.png')
+                parent=self.gui, cancelable=True, icon='lt.png', cancel_confirm_msg=_(
+                    'Aborting this operation means that only some books will be copied'
+                    ' and resuming a partial copy is not supported. Are you sure you want to abort?'))
 
         def progress(idx, title):
             self.pd.set_msg(title)
