@@ -9,22 +9,45 @@ import os
 from collections import OrderedDict
 from functools import partial
 
-from qt.core import (Qt, QComboBox, QLabel, QSpinBox, QDoubleSpinBox,
-        QDateTime, QGroupBox, QVBoxLayout, QSizePolicy, QGridLayout, QUrl,
-        QSpacerItem, QIcon, QCheckBox, QWidget, QHBoxLayout, QLineEdit,
-        QMessageBox, QToolButton, QPlainTextEdit, QApplication, QStyle, QDialog)
+from qt.core import (
+    QApplication,
+    QCheckBox,
+    QComboBox,
+    QDialog,
+    QDoubleSpinBox,
+    QGridLayout,
+    QGroupBox,
+    QHBoxLayout,
+    QIcon,
+    QLabel,
+    QLineEdit,
+    QMessageBox,
+    QPlainTextEdit,
+    QSizePolicy,
+    QSpacerItem,
+    QSpinBox,
+    QStyle,
+    Qt,
+    QToolButton,
+    QUrl,
+    QVBoxLayout,
+    QWidget,
+)
 
 from calibre.ebooks.metadata import title_sort
-from calibre.utils.date import qt_to_dt, now, as_local_time, as_utc, internal_iso_format_string
-from calibre.gui2.complete2 import EditWithComplete as EWC
+from calibre.gui2 import UNDEFINED_QDATETIME, elided_text, error_dialog, gprefs
 from calibre.gui2.comments_editor import Editor as CommentsEditor
-from calibre.gui2 import UNDEFINED_QDATETIME, error_dialog, elided_text, gprefs
+from calibre.gui2.complete2 import EditWithComplete as EWC
 from calibre.gui2.dialogs.tag_editor import TagEditor
-from calibre.utils.config import tweaks
-from calibre.utils.icu import sort_key
-from calibre.library.comments import comments_to_html
 from calibre.gui2.library.delegates import ClearingDoubleSpinBox, ClearingSpinBox
-from calibre.gui2.widgets2 import RatingEditor, DateTimeEdit as DateTimeEditBase
+from calibre.gui2.markdown_editor import Editor as MarkdownEditor
+from calibre.gui2.widgets2 import DateTimeEdit as DateTimeEditBase
+from calibre.gui2.widgets2 import RatingEditor
+from calibre.library.comments import comments_to_html
+from calibre.utils.config import tweaks
+from calibre.utils.date import as_local_time, as_utc, internal_iso_format_string, is_date_undefined, now, qt_from_dt, qt_to_dt
+from calibre.utils.icu import lower as icu_lower
+from calibre.utils.icu import sort_key
 
 
 class EditWithComplete(EWC):
@@ -46,7 +69,7 @@ def label_string(txt):
         try:
             if txt[0].isalnum():
                 return '&' + txt
-        except:
+        except Exception:
             pass
     return txt
 
@@ -72,12 +95,20 @@ class Base:
         try:
             self.widgets[0].setToolTip(description)
             self.widgets[1].setToolTip(description)
-        except:
+        except Exception:
             try:
                 self.widgets[1].setToolTip(description)
-            except:
+            except Exception:
                 pass
 
+    @property
+    def field_name(self) -> str:
+        return self.db.field_metadata.label_to_key(self.col_metadata['label'], prefer_custom=True)
+
+    @property
+    def hierarchy_separator(self) -> str:
+        return '.' if self.field_name in self.db.new_api.pref('categories_using_hierarchy', default=()) else ''
+
     def finish_ui_setup(self, parent, edit_widget):
         self.was_none = False
         w = QWidget(parent)
@@ -134,6 +165,27 @@ class Base:
     def connect_data_changed(self, slot):
         pass
 
+    def values_changed(self):
+        return self.getter() != self.initial_val and (self.getter() or self.initial_val)
+
+    def edit(self):
+        if self.values_changed():
+            d = _save_dialog(self.parent, _('Values changed'),
+                    _('You have changed the values. In order to use this '
+                       'editor, you must either discard or apply these '
+                       'changes. Apply changes?'))
+            if d == QMessageBox.StandardButton.Cancel:
+                return
+            if d == QMessageBox.StandardButton.Yes:
+                self.commit(self.book_id)
+                self.db.commit()
+                self.initial_val = self.current_val
+            else:
+                self.setter(self.initial_val)
+        from calibre.gui2.ui import get_gui
+        get_gui().do_tags_list_edit(None, self.key)
+        self.initialize(self.book_id)
+
 
 class SimpleText(Base):
 
@@ -324,10 +376,9 @@ class DateTimeEdit(DateTimeEditBase):
         DateTimeEditBase.focusOutEvent(self, x)
 
     def set_to_today(self):
-        self.setDateTime(now())
+        self.setDateTime(qt_from_dt(now()))
 
     def set_to_clear(self):
-        self.setDateTime(now())
         self.setDateTime(UNDEFINED_QDATETIME)
 
 
@@ -363,17 +414,25 @@ class DateTime(Base):
         self.clear_button.clicked.connect(dte.set_to_clear)
         self.clear_button.setToolTip(_('Clear {0}').format(self.col_metadata['name']))
         l.addWidget(self.clear_button)
+        self.connect_data_changed(self.set_tooltip)
+
+    def set_tooltip(self, val):
+        if is_date_undefined(val):
+            self.dte.setToolTip(get_tooltip(self.col_metadata, False))
+        else:
+            self.dte.setToolTip(get_tooltip(self.col_metadata, False) + '\n' +
+                                _('Exact time: {}').format(as_local_time(qt_to_dt(val))))
 
     def setter(self, val):
         if val is None:
             val = self.dte.minimumDateTime()
         else:
-            val = QDateTime(val)
+            val = qt_from_dt(val)
         self.dte.setDateTime(val)
 
     def getter(self):
         val = self.dte.dateTime()
-        if val <= UNDEFINED_QDATETIME:
+        if is_date_undefined(val):
             val = None
         else:
             val = qt_to_dt(val)
@@ -436,18 +495,65 @@ class Comments(Base):
         self.signals_to_disconnect.append(self._tb.data_changed)
 
 
+class Markdown(Base):
+
+    def setup_ui(self, parent):
+        self._box = QGroupBox(parent)
+        self._box.setTitle(label_string(self.col_metadata['name']))
+        self._layout = QVBoxLayout()
+        self._tb = MarkdownEditor(self._box)
+        self._tb.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)
+        # self._tb.setTabChangesFocus(True)
+        self._layout.addWidget(self._tb)
+        self._box.setLayout(self._layout)
+        self.widgets = [self._box]
+
+    def initialize(self, book_id):
+        path = self.db.abspath(book_id, index_is_id=True)
+        if path:
+            self._tb.set_base_url(QUrl.fromLocalFile(os.path.join(path, 'metadata.html')))
+        return super().initialize(book_id)
+
+    def setter(self, val):
+        self._tb.markdown = str(val or '').strip()
+
+    def getter(self):
+        val = self._tb.markdown.strip()
+        if not val:
+            val = None
+        return val
+
+    @property
+    def tab(self):
+        return self._tb.tab
+
+    @tab.setter
+    def tab(self, val):
+        self._tb.tab = val
+
+    def connect_data_changed(self, slot):
+        self._tb.editor.textChanged.connect(slot)
+        self.signals_to_disconnect.append(self._tb.editor.textChanged)
+
+
 class MultipleWidget(QWidget):
 
-    def __init__(self, parent):
+    def __init__(self, parent, only_manage_items=False, widget=EditWithComplete, name=None):
         QWidget.__init__(self, parent)
         layout = QHBoxLayout()
         layout.setSpacing(5)
         layout.setContentsMargins(0, 0, 0, 0)
 
-        self.tags_box = EditWithComplete(parent)
-        layout.addWidget(self.tags_box, stretch=1000)
+        self.edit_widget = widget(parent)
+        layout.addWidget(self.edit_widget, stretch=1000)
         self.editor_button = QToolButton(self)
-        self.editor_button.setToolTip(_('Open Item editor. If CTRL or SHIFT is pressed, open Manage items'))
+        if name is None:
+            name = _('items')
+        if only_manage_items:
+            self.editor_button.setToolTip(_('Open the Manage {} window').format(name))
+        else:
+            self.editor_button.setToolTip(_('Open the {0} editor. If Ctrl or Shift '
+                                            'is pressed, open the Manage {0} window').format(name))
         self.editor_button.setIcon(QIcon.ic('chapters.png'))
         layout.addWidget(self.editor_button)
         self.setLayout(layout)
@@ -456,34 +562,38 @@ class MultipleWidget(QWidget):
         return self.editor_button
 
     def update_items_cache(self, values):
-        self.tags_box.update_items_cache(values)
+        self.edit_widget.update_items_cache(values)
 
     def clear(self):
-        self.tags_box.clear()
+        self.edit_widget.clear()
 
     def setEditText(self):
-        self.tags_box.setEditText()
+        self.edit_widget.setEditText()
 
     def addItem(self, itm):
-        self.tags_box.addItem(itm)
+        self.edit_widget.addItem(itm)
 
     def set_separator(self, sep):
-        self.tags_box.set_separator(sep)
+        self.edit_widget.set_separator(sep)
+
+    def set_hierarchy_separator(self, sep):
+        if hasattr(self.edit_widget, 'set_hierarchy_separator'):
+            self.edit_widget.set_hierarchy_separator(sep)
 
     def set_add_separator(self, sep):
-        self.tags_box.set_add_separator(sep)
+        self.edit_widget.set_add_separator(sep)
 
     def set_space_before_sep(self, v):
-        self.tags_box.set_space_before_sep(v)
+        self.edit_widget.set_space_before_sep(v)
 
     def setSizePolicy(self, v1, v2):
-        self.tags_box.setSizePolicy(v1, v2)
+        self.edit_widget.setSizePolicy(v1, v2)
 
     def setText(self, v):
-        self.tags_box.setText(v)
+        self.edit_widget.setText(v)
 
     def text(self):
-        return self.tags_box.text()
+        return self.edit_widget.text()
 
 
 def _save_dialog(parent, title, msg, det_msg=''):
@@ -503,20 +613,19 @@ class Text(Base):
         self.parent = parent
 
         if self.col_metadata['is_multiple']:
-            w = MultipleWidget(parent)
+            w = MultipleWidget(parent, name=self.col_metadata['name'])
             w.set_separator(self.sep['ui_to_list'])
             if self.sep['ui_to_list'] == '&':
                 w.set_space_before_sep(True)
                 w.set_add_separator(tweaks['authors_completer_append_separator'])
             w.get_editor_button().clicked.connect(self.edit)
-            w.setSizePolicy(QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Fixed)
-            self.set_to_undefined = w.clear
         else:
-            w = EditWithComplete(parent)
+            w = MultipleWidget(parent, only_manage_items=True, name=self.col_metadata['name'])
             w.set_separator(None)
-            w.setSizeAdjustPolicy(QComboBox.SizeAdjustPolicy.AdjustToMinimumContentsLengthWithIcon)
-            w.setMinimumContentsLength(25)
-            self.set_to_undefined = w.clearEditText
+            w.get_editor_button().clicked.connect(super().edit)
+        w.set_hierarchy_separator(self.hierarchy_separator)
+        w.setSizePolicy(QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Fixed)
+        self.set_to_undefined = w.clear
         self.widgets = [QLabel(label_string(self.col_metadata['name']), parent)]
         self.finish_ui_setup(parent, lambda parent: w)
 
@@ -545,13 +654,12 @@ class Text(Base):
             self.editor.setText(self.sep['list_to_ui'].join(val))
 
     def getter(self):
+        val = str(self.editor.text()).strip()
         if self.col_metadata['is_multiple']:
-            val = str(self.editor.text()).strip()
             ans = [x.strip() for x in val.split(self.sep['ui_to_list']) if x.strip()]
             if not ans:
                 ans = None
             return ans
-        val = str(self.editor.currentText()).strip()
         if not val:
             val = None
         return val
@@ -582,10 +690,7 @@ class Text(Base):
                 self.setter(d.tags)
 
     def connect_data_changed(self, slot):
-        if self.col_metadata['is_multiple']:
-            s = self.editor.tags_box.currentTextChanged
-        else:
-            s = self.editor.currentTextChanged
+        s = self.editor.edit_widget.currentTextChanged
         s.connect(slot)
         self.signals_to_disconnect.append(s)
 
@@ -593,14 +698,19 @@ class Text(Base):
 class Series(Base):
 
     def setup_ui(self, parent):
-        w = EditWithComplete(parent, sort_func=title_sort)
+        self.parent = parent
+        self.key = self.db.field_metadata.label_to_key(self.col_metadata['label'],
+                                                       prefer_custom=True)
+        w = MultipleWidget(parent, only_manage_items=True, name=self.col_metadata['name'])
+        w.get_editor_button().clicked.connect(self.edit)
+        w.setSizePolicy(QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Fixed)
+        self.set_to_undefined = w.clear
         w.set_separator(None)
-        w.setSizeAdjustPolicy(QComboBox.SizeAdjustPolicy.AdjustToMinimumContentsLengthWithIcon)
-        w.setMinimumContentsLength(25)
-        self.name_widget = w
+        w.set_hierarchy_separator(self.hierarchy_separator)
+        self.name_widget = w.edit_widget
         self.widgets = [QLabel(label_string(self.col_metadata['name']), parent)]
         self.finish_ui_setup(parent, lambda parent: w)
-        w.editTextChanged.connect(self.series_changed)
+        self.name_widget.editTextChanged.connect(self.series_changed)
 
         w = QLabel(label_string(self.col_metadata['name'])+_(' index'), parent)
         w.setToolTip(get_tooltip(self.col_metadata, add_index=True))
@@ -618,6 +728,7 @@ class Series(Base):
         self.idx_widget.setValue(1.0)
 
     def initialize(self, book_id):
+        self.book_id = book_id
         values = list(self.db.all_custom(num=self.col_id))
         values.sort(key=sort_key)
         val = self.db.get_custom(book_id, num=self.col_id, index_is_id=True)
@@ -650,6 +761,10 @@ class Series(Base):
                                                      num=self.col_id)
         self.idx_widget.setValue(s_index)
 
+    def values_changed(self):
+        val, s_index = self.current_val
+        return val != self.initial_val or s_index != self.initial_index
+
     @property
     def current_val(self):
         val, s_index = self.gui_val
@@ -680,14 +795,22 @@ class Enumeration(Base):
 
     def setup_ui(self, parent):
         self.parent = parent
+        self.key = self.db.field_metadata.label_to_key(self.col_metadata['label'],
+                                                       prefer_custom=True)
+        w = MultipleWidget(parent, only_manage_items=True, widget=QComboBox, name=self.col_metadata['name'])
+        w.set_hierarchy_separator(self.hierarchy_separator)
+        w.get_editor_button().clicked.connect(self.edit)
+        w.setSizePolicy(QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Fixed)
         self.widgets = [QLabel(label_string(self.col_metadata['name']), parent)]
-        self.finish_ui_setup(parent, QComboBox)
+        self.finish_ui_setup(parent, lambda parent: w)
+        self.editor = w.edit_widget
         vals = self.col_metadata['display']['enum_values']
         self.editor.addItem('')
         for v in vals:
             self.editor.addItem(v)
 
     def initialize(self, book_id):
+        self.book_id = book_id
         val = self.db.get_custom(book_id, num=self.col_id, index_is_id=True)
         val = self.normalize_db_val(val)
         idx = self.editor.findText(val)
@@ -700,7 +823,7 @@ class Enumeration(Base):
 
             idx = 0
         self.editor.setCurrentIndex(idx)
-        self.initial_val = self.current_val
+        self.initial_val = val
 
     def setter(self, val):
         self.editor.setCurrentIndex(self.editor.findText(val))
@@ -731,18 +854,20 @@ def comments_factory(db, key, parent):
     ctype = fm.get('display', {}).get('interpret_as', 'html')
     if ctype == 'short-text':
         return SimpleText(db, key, parent)
-    if ctype in ('long-text', 'markdown'):
+    if ctype == 'long-text':
         return LongText(db, key, parent)
+    if ctype == 'markdown':
+        return Markdown(db, key, parent)
     return Comments(db, key, parent)
 
 
 widgets = {
-        'bool' : Bool,
-        'rating' : Rating,
+        'bool': Bool,
+        'rating': Rating,
         'int': Int,
         'float': Float,
         'datetime': DateTime,
-        'text' : Text,
+        'text': Text,
         'comments': comments_factory,
         'series': Series,
         'enumeration': Enumeration
@@ -777,7 +902,12 @@ def get_field_list(db, use_defaults=False, pref_data_override=None):
         for k in fields:
             if k not in result:
                 result[k] = True
-        return [(k,v) for k,v in result.items()]
+        return list(result.items())
+
+
+def get_custom_columns_to_display_in_editor(db):
+    return [k[0] for k in
+        get_field_list(db, use_defaults=db.prefs['edit_metadata_ignore_display_order']) if k[1]]
 
 
 def populate_metadata_page(layout, db, book_id, bulk=False, two_column=False, parent=None):
@@ -792,7 +922,7 @@ def populate_metadata_page(layout, db, book_id, bulk=False, two_column=False, pa
     fm = db.field_metadata
 
     # Get list of all non-composite custom fields. We must make widgets for these
-    cols = [k[0] for k in get_field_list(db, use_defaults=db.prefs['edit_metadata_ignore_display_order']) if k[1]]
+    cols = get_custom_columns_to_display_in_editor(db)
     # This deals with the historical behavior where comments fields go to the
     # bottom, starting on the left hand side. If a comment field is moved to
     # somewhere else then it isn't moved to either side.
@@ -847,6 +977,9 @@ def populate_metadata_page(layout, db, book_id, bulk=False, two_column=False, pa
         if is_comments:
             layout.addLayout(l, row, column, layout_rows_for_comments, 1)
             layout.setColumnStretch(column, 100)
+            # All multiline fields should grow with the window
+            for i in range(layout_rows_for_comments):
+                layout.setRowStretch(row + i, 100)
             row += layout_rows_for_comments
         else:
             layout.addLayout(l, row, column, 1, 1)
@@ -935,12 +1068,12 @@ class BulkBase(Base):
         if is_bool:
             self.set_no_button = QToolButton(parent)
             self.set_no_button.setIcon(QIcon.ic('list_remove.png'))
-            self.set_no_button.clicked.connect(lambda:self.main_widget.setCurrentIndex(1))
+            self.set_no_button.clicked.connect(lambda: self.main_widget.setCurrentIndex(1))
             self.set_no_button.setToolTip(_('Set {0} to No').format(self.col_metadata['name']))
             l.insertWidget(1, self.set_no_button)
             self.set_yes_button = QToolButton(parent)
             self.set_yes_button.setIcon(QIcon.ic('ok.png'))
-            self.set_yes_button.clicked.connect(lambda:self.main_widget.setCurrentIndex(0))
+            self.set_yes_button.clicked.connect(lambda: self.main_widget.setCurrentIndex(0))
             self.set_yes_button.setToolTip(_('Set {0} to Yes').format(self.col_metadata['name']))
             l.insertWidget(1, self.set_yes_button)
         if add_edit_tags_button[0]:
@@ -970,6 +1103,8 @@ class BulkBase(Base):
         l.setContentsMargins(0, 0, 0, 0)
         w.setLayout(l)
         self.main_widget = main_widget_class(w)
+        if (hs := self.hierarchy_separator) and hasattr(self.main_widget, 'set_hierarchy_separator'):
+            self.main_widget.set_hierarchy_separator(hs)
         l.addWidget(self.main_widget)
         l.setStretchFactor(self.main_widget, 10)
         self.a_c_checkbox = QCheckBox(_('Apply changes'), w)
@@ -1158,13 +1293,13 @@ class BulkDateTime(BulkBase):
         if val is None:
             val = self.main_widget.minimumDateTime()
         else:
-            val = QDateTime(val)
+            val = qt_from_dt(val)
         self.main_widget.setDateTime(val)
         self.ignore_change_signals = False
 
     def getter(self):
         val = self.main_widget.dateTime()
-        if val <= UNDEFINED_QDATETIME:
+        if is_date_undefined(val):
             val = None
         else:
             val = qt_to_dt(val)
@@ -1212,12 +1347,12 @@ class BulkSeries(BulkBase):
         self.series_start_number = QDoubleSpinBox(parent)
         self.series_start_number.setMinimum(0.0)
         self.series_start_number.setMaximum(9999999.0)
-        self.series_start_number.setProperty("value", 1.0)
+        self.series_start_number.setProperty('value', 1.0)
         layout.addWidget(self.series_start_number)
         self.series_increment = QDoubleSpinBox(parent)
         self.series_increment.setMinimum(0.00)
         self.series_increment.setMaximum(99999.0)
-        self.series_increment.setProperty("value", 1.0)
+        self.series_increment.setProperty('value', 1.0)
         self.series_increment.setToolTip('<p>' + _(
             'The amount by which to increment the series number '
             'for successive books. Only applicable when using '
@@ -1432,7 +1567,7 @@ class BulkText(BulkBase):
             self.main_widget.setMinimumContentsLength(25)
         self.ignore_change_signals = False
         self.parent = parent
-        self.finish_ui_setup(parent, add_edit_tags_button=(is_tags,self.edit_add))
+        self.finish_ui_setup(parent, add_edit_tags_button=(is_tags, self.edit_add))
 
     def set_to_undefined(self):
         self.main_widget.clearEditText()
@@ -1514,12 +1649,12 @@ class BulkText(BulkBase):
 
 
 bulk_widgets = {
-        'bool' : BulkBool,
-        'rating' : BulkRating,
+        'bool': BulkBool,
+        'rating': BulkRating,
         'int': BulkInt,
         'float': BulkFloat,
         'datetime': BulkDateTime,
-        'text' : BulkText,
+        'text': BulkText,
         'series': BulkSeries,
         'enumeration': BulkEnumeration,
 }
