@@ -54,41 +54,51 @@ from sys import stderr
 
 from polyglot.builtins import string_or_bytes
 
-__all__ = ["SMTPException", "SMTPServerDisconnected", "SMTPResponseException",
-           "SMTPSenderRefused", "SMTPRecipientsRefused", "SMTPDataError",
-           "SMTPConnectError", "SMTPHeloError", "SMTPAuthenticationError",
-           "quoteaddr", "quotedata", "SMTP"]
+__all__ = [
+    'SMTP',
+    'SMTPAuthenticationError',
+    'SMTPConnectError',
+    'SMTPDataError',
+    'SMTPException',
+    'SMTPHeloError',
+    'SMTPRecipientsRefused',
+    'SMTPResponseException',
+    'SMTPSenderRefused',
+    'SMTPServerDisconnected',
+    'quoteaddr',
+    'quotedata',
+]
 
 SMTP_PORT = 25
 SMTP_SSL_PORT = 465
-CRLF = "\r\n"
+CRLF = '\r\n'
 _MAXLINE = 8192  # more than 8 times larger than RFC 821, 4.5.3
 
-OLDSTYLE_AUTH = re.compile(r"auth=(.*)", re.I)
+OLDSTYLE_AUTH = re.compile(r'auth=(.*)', re.I)
 
 
 # Exception classes used by this module.
 class SMTPException(Exception):
-    """Base class for all exceptions raised by this module."""
+    '''Base class for all exceptions raised by this module.'''
 
 
 class SMTPServerDisconnected(SMTPException):
-    """Not connected to any SMTP server.
+    '''Not connected to any SMTP server.
 
     This exception is raised when the server unexpectedly disconnects,
     or when an attempt is made to use the SMTP instance before
     connecting it to a server.
-    """
+    '''
 
 
 class SMTPResponseException(SMTPException):
-    """Base class for all exceptions that include an SMTP error code.
+    '''Base class for all exceptions that include an SMTP error code.
 
     These exceptions are generated in some instances when the SMTP
     server returns an error code.  The error code is stored in the
     `smtp_code' attribute of the error, and the `smtp_error' attribute
     is set to the error message.
-    """
+    '''
 
     def __init__(self, code, msg):
         self.smtp_code = code
@@ -97,11 +107,11 @@ class SMTPResponseException(SMTPException):
 
 
 class SMTPSenderRefused(SMTPResponseException):
-    """Sender address refused.
+    '''Sender address refused.
 
     In addition to the attributes set by on all SMTPResponseException
     exceptions, this sets `sender' to the string that the SMTP refused.
-    """
+    '''
 
     def __init__(self, code, msg, sender):
         self.smtp_code = code
@@ -111,12 +121,12 @@ class SMTPSenderRefused(SMTPResponseException):
 
 
 class SMTPRecipientsRefused(SMTPException):
-    """All recipient addresses refused.
+    '''All recipient addresses refused.
 
     The errors for each recipient are accessible through the attribute
     'recipients', which is a dictionary of exactly the same sort as
     SMTP.sendmail() returns.
-    """
+    '''
 
     def __init__(self, recipients):
         self.recipients = recipients
@@ -124,30 +134,30 @@ class SMTPRecipientsRefused(SMTPException):
 
 
 class SMTPDataError(SMTPResponseException):
-    """The SMTP server didn't accept the data."""
+    '''The SMTP server didn't accept the data.'''
 
 
 class SMTPConnectError(SMTPResponseException):
-    """Error during connection establishment."""
+    '''Error during connection establishment.'''
 
 
 class SMTPHeloError(SMTPResponseException):
-    """The server refused our HELO reply."""
+    '''The server refused our HELO reply.'''
 
 
 class SMTPAuthenticationError(SMTPResponseException):
-    """Authentication error.
+    '''Authentication error.
 
     Most probably the server didn't accept the username/password
     combination provided.
-    """
+    '''
 
 
 def quoteaddr(addr):
-    """Quote a subset of the email addresses defined by RFC 821.
+    '''Quote a subset of the email addresses defined by RFC 821.
 
     Should be able to handle anything rfc822.parseaddr can handle.
-    """
+    '''
     m = (None, None)
     try:
         m = email.utils.parseaddr(addr)[1]
@@ -155,12 +165,12 @@ def quoteaddr(addr):
         pass
     if m == (None, None):  # Indicates parse failure or AttributeError
         # something weird here.. punt -ddm
-        return "<%s>" % addr
+        return f'<{addr}>'
     elif m is None:
         # the sender wants an empty return address
-        return "<>"
+        return '<>'
     else:
-        return "<%s>" % m
+        return f'<{m}>'
 
 
 def _addr_only(addrstring):
@@ -172,11 +182,11 @@ def _addr_only(addrstring):
 
 
 def quotedata(data):
-    """Quote data for email.
+    '''Quote data for email.
 
     Double leading '.', and change Unix newline '\\n', or Mac '\\r' into
     Internet CRLF end-of-line.
-    """
+    '''
     return re.sub(r'(?m)^\.', '..',
         re.sub(r'(?:\r\n|\n|\r(?!\n))', CRLF, data))
 
@@ -187,10 +197,10 @@ except ImportError:
     _have_ssl = False
 else:
     class SSLFakeFile:
-        """A fake file like object that really wraps a SSLObject.
+        '''A fake file like object that really wraps a SSLObject.
 
         It only supports what is needed in smtplib.
-        """
+        '''
 
         def __init__(self, sslobj):
             self.sslobj = sslobj
@@ -198,9 +208,9 @@ else:
         def readline(self, size=-1):
             if size < 0:
                 size = None
-            str = ""
+            str = ''
             chr = None
-            while chr != "\n":
+            while chr != '\n':
                 if size is not None and len(str) >= size:
                     break
                 chr = self.sslobj.read(1)
@@ -216,7 +226,7 @@ else:
 
 
 class SMTP:
-    """This class manages a connection to an SMTP or ESMTP server.
+    '''This class manages a connection to an SMTP or ESMTP server.
     SMTP Objects:
         SMTP objects have the following attributes:
             helo_resp
@@ -243,11 +253,11 @@ class SMTP:
         See each method's docstrings for details.  In general, there is a
         method of the same name to perform each SMTP command.  There is also a
         method called 'sendmail' that will do an entire mail transaction.
-        """
+        '''
     debuglevel = 0
     file = None
     helo_resp = None
-    ehlo_msg = "ehlo"
+    ehlo_msg = 'ehlo'
     ehlo_resp = None
     does_esmtp = 0
     default_port = SMTP_PORT
@@ -255,7 +265,7 @@ class SMTP:
     def __init__(self, host='', port=0, local_hostname=None,
                  timeout=socket._GLOBAL_DEFAULT_TIMEOUT,
                  debug_to=partial(print, file=stderr)):
-        """Initialize a new instance.
+        '''Initialize a new instance.
 
         If specified, `host' is the name of the remote host to which to
         connect.  If specified, `port' specifies the port to which to connect.
@@ -266,13 +276,13 @@ class SMTP:
         specifies where debug output is written to. By default it is written to
         sys.stderr. You should pass in a print function of your own to control
         where debug output is written.
-        """
+        '''
         self._host = host
         self.timeout = timeout
         self.debug = debug_to
         self.esmtp_features = {}
         if host:
-            (code, msg) = self.connect(host, port)
+            code, msg = self.connect(host, port)
             if code != 220:
                 raise SMTPConnectError(code, msg)
         if local_hostname is not None:
@@ -291,17 +301,17 @@ class SMTP:
                     addr = socket.gethostbyname(socket.gethostname())
                 except socket.gaierror:
                     pass
-                self.local_hostname = '[%s]' % addr
+                self.local_hostname = f'[{addr}]'
 
     def set_debuglevel(self, debuglevel):
-        """Set the debug output level.
+        '''Set the debug output level.
 
         A value of 0 means no debug logging. A value of 1 means all interaction
         with the server is logged except that long lines are truncated to 100
         characters and AUTH messages are censored. A value of 2 or higher means
         the complete session is logged.
 
-        """
+        '''
         self.debuglevel = debuglevel
 
     def _get_socket(self, host, port, timeout):
@@ -312,7 +322,7 @@ class SMTP:
         return socket.create_connection((host, port), timeout)
 
     def connect(self, host='localhost', port=0):
-        """Connect to a host on a given port.
+        '''Connect to a host on a given port.
 
         If the hostname ends with a colon (`:') followed by a number, and
         there is no port specified, that suffix will be stripped off and the
@@ -321,7 +331,7 @@ class SMTP:
         Note: This method is automatically invoked by __init__, if a host is
         specified during instantiation.
 
-        """
+        '''
         if not port and (host.find(':') == host.rfind(':')):
             i = host.rfind(':')
             if i >= 0:
@@ -329,20 +339,20 @@ class SMTP:
                 try:
                     port = int(port)
                 except ValueError:
-                    raise OSError("nonnumeric port")
+                    raise OSError('nonnumeric port')
         if not port:
             port = self.default_port
         if self.debuglevel > 0:
             self.debug('connect:', (host, port))
         self._host = host
         self.sock = self._get_socket(host, port, self.timeout)
-        (code, msg) = self.getreply()
+        code, msg = self.getreply()
         if self.debuglevel > 0:
-            self.debug("connect:", msg)
-        return (code, msg)
+            self.debug('connect:', msg)
+        return code, msg
 
     def send(self, str):
-        """Send `str' to the server."""
+        '''Send `str' to the server.'''
         if self.debuglevel > 0:
             raw = repr(str)
             self.debug('send:', raw)
@@ -355,16 +365,16 @@ class SMTP:
         else:
             raise SMTPServerDisconnected('please run connect() first')
 
-    def putcmd(self, cmd, args=""):
-        """Send a command to the server."""
-        if args == "":
+    def putcmd(self, cmd, args=''):
+        '''Send a command to the server.'''
+        if args == '':
             str = f'{cmd}{CRLF}'
         else:
             str = f'{cmd} {args}{CRLF}'
         self.send(str)
 
     def getreply(self):
-        """Get a reply from the server.
+        '''Get a reply from the server.
 
         Returns a tuple consisting of:
 
@@ -375,7 +385,7 @@ class SMTP:
             responses are converted to a single, multiline string).
 
         Raises SMTPServerDisconnected if end-of-file is reached.
-        """
+        '''
         resp = []
         if self.file is None:
             self.file = self.sock.makefile('rb')
@@ -384,14 +394,14 @@ class SMTP:
                 line = self.file.readline(_MAXLINE + 1)
             except OSError as e:
                 self.close()
-                raise SMTPServerDisconnected("Connection unexpectedly closed: " + str(e))
+                raise SMTPServerDisconnected('Connection unexpectedly closed: ' + str(e))
             if line == '':
                 self.close()
-                raise SMTPServerDisconnected("Connection unexpectedly closed")
+                raise SMTPServerDisconnected('Connection unexpectedly closed')
             if self.debuglevel > 0:
                 self.debug('reply:', repr(line))
             if len(line) > _MAXLINE:
-                raise SMTPResponseException(500, "Line too long.")
+                raise SMTPResponseException(500, 'Line too long.')
             resp.append(line[4:].strip())
             code = line[:3]
             # Check that the error code is syntactically correct.
@@ -402,47 +412,47 @@ class SMTP:
                 errcode = -1
                 break
             # Check if multiline response.
-            if line[3:4] != "-":
+            if line[3:4] != '-':
                 break
 
-        errmsg = "\n".join(resp)
+        errmsg = '\n'.join(resp)
         if self.debuglevel > 0:
             self.debug(f'reply: retcode ({errcode}); Msg: {errmsg}')
         return errcode, errmsg
 
-    def docmd(self, cmd, args=""):
-        """Send a command, and return its response code."""
+    def docmd(self, cmd, args=''):
+        '''Send a command, and return its response code.'''
         self.putcmd(cmd, args)
         return self.getreply()
 
     # std smtp commands
     def helo(self, name=''):
-        """SMTP 'helo' command.
+        '''SMTP 'helo' command.
         Hostname to send for this command defaults to the FQDN of the local
         host.
-        """
-        self.putcmd("helo", name or self.local_hostname)
-        (code, msg) = self.getreply()
+        '''
+        self.putcmd('helo', name or self.local_hostname)
+        code, msg = self.getreply()
         self.helo_resp = msg
-        return (code, msg)
+        return code, msg
 
     def ehlo(self, name=''):
-        """ SMTP 'ehlo' command.
+        ''' SMTP 'ehlo' command.
         Hostname to send for this command defaults to the FQDN of the local
         host.
-        """
+        '''
         self.esmtp_features = {}
         self.putcmd(self.ehlo_msg, name or self.local_hostname)
-        (code, msg) = self.getreply()
+        code, msg = self.getreply()
         # According to RFC1869 some (badly written)
         # MTA's will disconnect on an ehlo. Toss an exception if
         # that happens -ddm
         if code == -1 and len(msg) == 0:
             self.close()
-            raise SMTPServerDisconnected("Server not connected")
+            raise SMTPServerDisconnected('Server not connected')
         self.ehlo_resp = msg
         if code != 250:
-            return (code, msg)
+            return code, msg
         self.does_esmtp = 1
         # parse the ehlo response -ddm
         resp = self.ehlo_resp.split('\n')
@@ -457,8 +467,8 @@ class SMTP:
             auth_match = OLDSTYLE_AUTH.match(each)
             if auth_match:
                 # This doesn't remove duplicates, but that's no problem
-                self.esmtp_features["auth"] = self.esmtp_features.get("auth", "") \
-                        + " " + auth_match.groups(0)[0]
+                self.esmtp_features['auth'] = self.esmtp_features.get('auth', '') \
+                        + ' ' + auth_match.groups(0)[0]
                 continue
 
             # RFC 1869 requires a space between ehlo keyword and parameters.
@@ -467,90 +477,90 @@ class SMTP:
             # that the space isn't present if there are no parameters.
             m = re.match(r'(?P<feature>[A-Za-z0-9][A-Za-z0-9\-]*) ?', each)
             if m:
-                feature = m.group("feature").lower()
-                params = m.string[m.end("feature"):].strip()
-                if feature == "auth":
-                    self.esmtp_features[feature] = self.esmtp_features.get(feature, "") \
-                            + " " + params
+                feature = m.group('feature').lower()
+                params = m.string[m.end('feature'):].strip()
+                if feature == 'auth':
+                    self.esmtp_features[feature] = self.esmtp_features.get(feature, '') \
+                            + ' ' + params
                 else:
                     self.esmtp_features[feature] = params
-        return (code, msg)
+        return code, msg
 
     def has_extn(self, opt):
-        """Does the server support a given SMTP service extension?"""
+        '''Does the server support a given SMTP service extension?'''
         return opt.lower() in self.esmtp_features
 
     def help(self, args=''):
-        """SMTP 'help' command.
-        Returns help text from server."""
-        self.putcmd("help", args)
+        '''SMTP 'help' command.
+        Returns help text from server.'''
+        self.putcmd('help', args)
         return self.getreply()[1]
 
     def rset(self):
-        """SMTP 'rset' command -- resets session."""
-        return self.docmd("rset")
+        '''SMTP 'rset' command -- resets session.'''
+        return self.docmd('rset')
 
     def noop(self):
-        """SMTP 'noop' command -- doesn't do anything :>"""
-        return self.docmd("noop")
+        '''SMTP 'noop' command -- doesn't do anything :>'''
+        return self.docmd('noop')
 
     def mail(self, sender, options=[]):
-        """SMTP 'mail' command -- begins mail xfer session."""
+        '''SMTP 'mail' command -- begins mail xfer session.'''
         optionlist = ''
         if options and self.does_esmtp:
             optionlist = ' ' + ' '.join(options)
-        self.putcmd("mail", f"FROM:{quoteaddr(sender)}{optionlist}")
+        self.putcmd('mail', f'FROM:{quoteaddr(sender)}{optionlist}')
         return self.getreply()
 
     def rcpt(self, recip, options=[]):
-        """SMTP 'rcpt' command -- indicates 1 recipient for this mail."""
+        '''SMTP 'rcpt' command -- indicates 1 recipient for this mail.'''
         optionlist = ''
         if options and self.does_esmtp:
             optionlist = ' ' + ' '.join(options)
-        self.putcmd("rcpt", f"TO:{quoteaddr(recip)}{optionlist}")
+        self.putcmd('rcpt', f'TO:{quoteaddr(recip)}{optionlist}')
         return self.getreply()
 
     def data(self, msg):
-        """SMTP 'DATA' command -- sends message data to server.
+        '''SMTP 'DATA' command -- sends message data to server.
 
         Automatically quotes lines beginning with a period per rfc821.
         Raises SMTPDataError if there is an unexpected reply to the
         DATA command; the return value from this method is the final
         response code received when the all data is sent.
-        """
-        self.putcmd("data")
-        (code, repl) = self.getreply()
+        '''
+        self.putcmd('data')
+        code, repl = self.getreply()
         if self.debuglevel > 0:
-            self.debug("data:", (code, repl))
+            self.debug('data:', (code, repl))
         if code != 354:
             raise SMTPDataError(code, repl)
         else:
             q = quotedata(msg)
             if q[-2:] != CRLF:
                 q = q + CRLF
-            q = q + "." + CRLF
+            q = q + '.' + CRLF
             self.send(q)
-            (code, msg) = self.getreply()
-            if self.debuglevel > 0 :
-                self.debug("data:", (code, msg))
-            return (code, msg)
+            code, msg = self.getreply()
+            if self.debuglevel > 0:
+                self.debug('data:', (code, msg))
+            return code, msg
 
     def verify(self, address):
-        """SMTP 'verify' command -- checks for address validity."""
-        self.putcmd("vrfy", _addr_only(address))
+        '''SMTP 'verify' command -- checks for address validity.'''
+        self.putcmd('vrfy', _addr_only(address))
         return self.getreply()
     # a.k.a.
     vrfy = verify
 
     def expn(self, address):
-        """SMTP 'expn' command -- expands a mailing list."""
-        self.putcmd("expn", _addr_only(address))
+        '''SMTP 'expn' command -- expands a mailing list.'''
+        self.putcmd('expn', _addr_only(address))
         return self.getreply()
 
     # some useful methods
 
     def ehlo_or_helo_if_needed(self):
-        """Call self.ehlo() and/or self.helo() if needed.
+        '''Call self.ehlo() and/or self.helo() if needed.
 
         If there has been no previous EHLO or HELO command this session, this
         method tries ESMTP EHLO first.
@@ -559,15 +569,15 @@ class SMTP:
 
          SMTPHeloError            The server didn't reply properly to
                                   the helo greeting.
-        """
+        '''
         if self.helo_resp is None and self.ehlo_resp is None:
             if not (200 <= self.ehlo()[0] <= 299):
-                (code, resp) = self.helo()
+                code, resp = self.helo()
                 if not (200 <= code <= 299):
                     raise SMTPHeloError(code, resp)
 
     def login(self, user, password):
-        """Log in on an SMTP server that requires authentication.
+        '''Log in on an SMTP server that requires authentication.
 
         The arguments are:
             - user:     The user name to authenticate with.
@@ -586,29 +596,29 @@ class SMTP:
                                   password combination.
          SMTPException            No suitable authentication method was
                                   found.
-        """
+        '''
 
         def encode_cram_md5(challenge, user, password):
             challenge = base64.decodestring(challenge)
             if isinstance(password, str):  # Added by Kovid, see http://bugs.python.org/issue5285
                 password = password.encode('utf-8')
-            response = user + " " + hmac.HMAC(password, challenge).hexdigest()
-            return encode_base64(response, eol="")
+            response = user + ' ' + hmac.HMAC(password, challenge).hexdigest()
+            return encode_base64(response, eol='')
 
         def encode_plain(user, password):
-            return encode_base64(f"\0{user}\0{password}", eol="")
+            return encode_base64(f'\0{user}\0{password}', eol='')
 
-        AUTH_PLAIN = "PLAIN"
-        AUTH_CRAM_MD5 = "CRAM-MD5"
-        AUTH_LOGIN = "LOGIN"
+        AUTH_PLAIN = 'PLAIN'
+        AUTH_CRAM_MD5 = 'CRAM-MD5'
+        AUTH_LOGIN = 'LOGIN'
 
         self.ehlo_or_helo_if_needed()
 
-        if not self.has_extn("auth"):
-            raise SMTPException("SMTP AUTH extension not supported by server.")
+        if not self.has_extn('auth'):
+            raise SMTPException('SMTP AUTH extension not supported by server.')
 
         # Authentication methods the server supports:
-        authlist = self.esmtp_features["auth"].split()
+        authlist = self.esmtp_features['auth'].split()
 
         # List of authentication methods we support: from preferred to
         # less preferred methods. Except for the purpose of testing the weaker
@@ -623,30 +633,30 @@ class SMTP:
                 break
 
         if authmethod == AUTH_CRAM_MD5:
-            (code, resp) = self.docmd("AUTH", AUTH_CRAM_MD5)
+            code, resp = self.docmd('AUTH', AUTH_CRAM_MD5)
             if code == 503:
                 # 503 == 'Error: already authenticated'
-                return (code, resp)
-            (code, resp) = self.docmd(encode_cram_md5(resp, user, password))
+                return code, resp
+            code, resp = self.docmd(encode_cram_md5(resp, user, password))
         elif authmethod == AUTH_PLAIN:
-            (code, resp) = self.docmd("AUTH",
-                AUTH_PLAIN + " " + encode_plain(user, password))
+            code, resp = self.docmd('AUTH',
+                AUTH_PLAIN + ' ' + encode_plain(user, password))
         elif authmethod == AUTH_LOGIN:
-            (code, resp) = self.docmd("AUTH",
-                "{} {}".format(AUTH_LOGIN, encode_base64(user, eol="")))
+            code, resp = self.docmd('AUTH',
+                '{} {}'.format(AUTH_LOGIN, encode_base64(user, eol='')))
             if code != 334:
                 raise SMTPAuthenticationError(code, resp)
-            (code, resp) = self.docmd(encode_base64(password, eol=""))
+            code, resp = self.docmd(encode_base64(password, eol=''))
         elif authmethod is None:
-            raise SMTPException("No suitable authentication method found.")
+            raise SMTPException('No suitable authentication method found.')
         if code not in (235, 503):
             # 235 == 'Authentication successful'
             # 503 == 'Error: already authenticated'
             raise SMTPAuthenticationError(code, resp)
-        return (code, resp)
+        return code, resp
 
     def starttls(self, context=None):
-        """Puts the connection to the SMTP server into TLS mode.
+        '''Puts the connection to the SMTP server into TLS mode.
 
         If there has been no previous EHLO or HELO command this session, this
         method tries ESMTP EHLO first.
@@ -661,14 +671,14 @@ class SMTP:
 
          SMTPHeloError            The server didn't reply properly to
                                   the helo greeting.
-        """
+        '''
         self.ehlo_or_helo_if_needed()
-        if not self.has_extn("starttls"):
-            raise SMTPException("STARTTLS extension not supported by server.")
-        (resp, reply) = self.docmd("STARTTLS")
+        if not self.has_extn('starttls'):
+            raise SMTPException('STARTTLS extension not supported by server.')
+        resp, reply = self.docmd('STARTTLS')
         if resp == 220:
             if not _have_ssl:
-                raise RuntimeError("No SSL support included in this Python")
+                raise RuntimeError('No SSL support included in this Python')
             if context is None:
                 self.sock = ssl.wrap_socket(self.sock)
             else:
@@ -687,7 +697,7 @@ class SMTP:
             # 501 Syntax error (no parameters allowed)
             # 454 TLS not available due to temporary reason
             raise SMTPResponseException(resp, reply)
-        return (resp, reply)
+        return resp, reply
 
     def sendmail(self, from_addr, to_addrs, msg, mail_options=[],
                  rcpt_options=[]):
@@ -733,30 +743,30 @@ class SMTP:
          >>> tolist=["one@one.org","two@two.org","three@three.org","four@four.org"]
          >>> msg = '''\\
          ... From: Me@my.org
-         ... Subject: testin'...
+         ... Subject: testing...
          ...
          ... This is a test '''
          >>> s.sendmail("me@my.org",tolist,msg)
-         { "three@three.org" : ( 550 ,"User unknown" ) }
+         { "three@three.org" : (550, "User unknown") }
          >>> s.quit()
 
         In the above example, the message was accepted for delivery to three
         of the four addresses, and one was rejected, with the error code
         550.  If all addresses are accepted, then the method will return an
         empty dictionary.
-
         """
+
         self.ehlo_or_helo_if_needed()
         esmtp_opts = []
         if self.does_esmtp:
             # Hmmm? what's this? -ddm
             # self.esmtp_features['7bit']=""
             if self.has_extn('size'):
-                esmtp_opts.append("size=%d" % len(msg))
+                esmtp_opts.append(f'size={len(msg)}')
             for option in mail_options:
                 esmtp_opts.append(option)
 
-        (code, resp) = self.mail(from_addr, esmtp_opts)
+        code, resp = self.mail(from_addr, esmtp_opts)
         if code != 250:
             self.rset()
             raise SMTPSenderRefused(code, resp, from_addr)
@@ -764,14 +774,14 @@ class SMTP:
         if isinstance(to_addrs, string_or_bytes):
             to_addrs = [to_addrs]
         for each in to_addrs:
-            (code, resp) = self.rcpt(each, rcpt_options)
+            code, resp = self.rcpt(each, rcpt_options)
             if (code != 250) and (code != 251):
-                senderrs[each] = (code,resp)
+                senderrs[each] = (code, resp)
         if len(senderrs) == len(to_addrs):
             # the server refused all our recipients
             self.rset()
             raise SMTPRecipientsRefused(senderrs)
-        (code, resp) = self.data(msg)
+        code, resp = self.data(msg)
         if code != 250:
             self.rset()
             raise SMTPDataError(code, resp)
@@ -779,7 +789,7 @@ class SMTP:
         return senderrs
 
     def close(self):
-        """Close the connection to the SMTP server."""
+        '''Close the connection to the SMTP server.'''
         try:
             file = self.file
             self.file = None
@@ -792,8 +802,8 @@ class SMTP:
                 sock.close()
 
     def quit(self):
-        """Terminate the SMTP session."""
-        res = self.docmd("quit")
+        '''Terminate the SMTP session.'''
+        res = self.docmd('quit')
         # A new EHLO is required after reconnecting with connect()
         self.ehlo_resp = self.helo_resp = None
         self.esmtp_features = {}
@@ -805,13 +815,13 @@ class SMTP:
 if _have_ssl:
 
     class SMTP_SSL(SMTP):
-        """ This is a subclass derived from SMTP that connects over an SSL encrypted
+        ''' This is a subclass derived from SMTP that connects over an SSL encrypted
         socket (to use this class you need a socket module that was compiled with SSL
         support). If host is not specified, '' (the local host) is used. If port is
         omitted, the standard SMTP-over-SSL port (465) is used. keyfile and certfile
         are also optional - they can contain a PEM formatted private key and
         certificate chain file for the SSL connection.
-        """
+        '''
 
         default_port = SMTP_SSL_PORT
 
@@ -832,7 +842,7 @@ if _have_ssl:
             self.file = SSLFakeFile(new_socket)
             return new_socket
 
-    __all__.append("SMTP_SSL")
+    __all__.append('SMTP_SSL')
 
 #
 # LMTP extension
@@ -841,7 +851,7 @@ LMTP_PORT = 2003
 
 
 class LMTP(SMTP):
-    """LMTP - Local Mail Transfer Protocol
+    '''LMTP - Local Mail Transfer Protocol
 
     The LMTP protocol, which is very similar to ESMTP, is heavily based
     on the standard SMTP client. It's common to use Unix sockets for LMTP,
@@ -851,16 +861,16 @@ class LMTP(SMTP):
 
     Authentication is supported, using the regular SMTP mechanism. When
     using a Unix socket, LMTP generally don't support or require any
-    authentication, but your mileage might vary."""
+    authentication, but your mileage might vary.'''
 
-    ehlo_msg = "lhlo"
+    ehlo_msg = 'lhlo'
 
     def __init__(self, host='', port=LMTP_PORT, local_hostname=None):
-        """Initialize a new instance."""
+        '''Initialize a new instance.'''
         SMTP.__init__(self, host, port, local_hostname)
 
     def connect(self, host='localhost', port=0):
-        """Connect to the LMTP daemon, on either a Unix or a TCP socket."""
+        '''Connect to the LMTP daemon, on either a Unix or a TCP socket.'''
         if host[0] != '/':
             return SMTP.connect(self, host, port)
 
@@ -875,10 +885,10 @@ class LMTP(SMTP):
                 self.sock.close()
             self.sock = None
             raise
-        (code, msg) = self.getreply()
+        code, msg = self.getreply()
         if self.debuglevel > 0:
-            self.debug("connect:", msg)
-        return (code, msg)
+            self.debug('connect:', msg)
+        return code, msg
 
 
 # Test the sendmail method, which tests most of the others.
@@ -887,19 +897,19 @@ if __name__ == '__main__':
     import sys
 
     def prompt(prompt):
-        sys.stdout.write(prompt + ": ")
+        sys.stdout.write(prompt + ': ')
         return sys.stdin.readline().strip()
 
-    fromaddr = prompt("From")
-    toaddrs = prompt("To").split(',')
-    print("Enter message, end with ^D:")
+    fromaddr = prompt('From')
+    toaddrs = prompt('To').split(',')
+    print('Enter message, end with ^D:')
     msg = ''
     while 1:
         line = sys.stdin.readline()
         if not line:
             break
         msg = msg + line
-    print("Message length is %d" % len(msg))
+    print(f'Message length is {len(msg)}')
 
     server = SMTP('localhost')
     server.set_debuglevel(1)
